@@ -24,13 +24,14 @@
 
 import numpy as np
 import pandas as pd
+from pyspark.sql.functions import pandas_udf, PandasUDFType
+from pyspark.sql.group import GroupedData
+from pyspark.sql.types import StructField, StructType, StringType, FloatType, TimestampType, IntegerType
 
+from cerebralcortex.algorithms.utils.util import update_metadata
 from cerebralcortex.core.datatypes import DataStream
 from cerebralcortex.core.metadata_manager.stream.metadata import Metadata
-from pyspark.sql.functions import pandas_udf, PandasUDFType
-from pyspark.sql.types import StructField, StructType, StringType, FloatType, TimestampType, IntegerType
-from pyspark.sql.group import GroupedData
-from cerebralcortex.algorithms.utils.util import update_metadata
+
 
 def glucose_var(ds):
     """
@@ -555,11 +556,13 @@ def glucose_var(ds):
         return pdf
 
     # check if datastream object contains grouped type of DataFrame
-    if not isinstance(ds._data, GroupedData):
-        raise Exception(
-            "DataStream object is not grouped data type. Please use 'window' operation on datastream object before running this algorithm")
+    # if not isinstance(ds._data, GroupedData):
+    #     raise Exception(
+    #         "DataStream object is not grouped data type. Please use 'window' operation on datastream object before running this algorithm")
 
-    data = ds._data.apply(get_all_metrics)
+
+
+    data = ds._data.groupBy(["user", "version"]).apply(get_all_metrics)
 
     results = DataStream(data=data, metadata=Metadata())
     metadta = update_metadata(stream_metadata=results.metadata,
