@@ -24,7 +24,6 @@
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import hashlib
-import json
 import random
 import re
 import string
@@ -36,6 +35,8 @@ from typing import List
 import jwt
 from pytz import timezone
 
+from cerebralcortex.core.data_manager.sql.orm_models import User
+
 
 class UserHandler():
 
@@ -67,37 +68,18 @@ class UserHandler():
         user_uuid = str(uuid.uuid3(uuid.NAMESPACE_DNS, user_uuid))
         if encrypt_password:
             user_password = self.encrypt_user_password(user_password)
-        qry = "INSERT IGNORE INTO " + self.userTable + " (user_id, username, password, study_name, user_role, user_metadata,user_settings) VALUES(%s, %s, %s, %s, %s, %s, %s)"
-        vals = str(user_uuid), str(username), str(user_password), str(self.study_name), str(user_role), json.dumps(user_metadata), json.dumps(user_settings)
+
+        user = User(user_id=user_uuid, username=username, password=user_password, study_name=self.study_name,
+                    token="", token_issued=datetime.now(), token_expiry=datetime.now(), user_role=user_role,
+                    user_settings=user_settings, user_metadata=user_metadata)
 
         try:
-            self.execute(qry, vals, commit=True)
+            self.session.add(user)
+            self.session.commit()
             return True
         except Exception as e:
             raise Exception(e)
 
-
-    def delete_user(self, username:str):
-        """
-        Delete a user record in SQL table
-
-        Args:
-            username: username of a user that needs to be deleted
-        Returns:
-            bool: if user is successfully removed
-        Raises:
-            ValueError: if username param is empty or None
-            Exception: if sql query fails
-        """
-        if not username:
-            raise ValueError("username cannot be empty/None.")
-        qry = "delete from "+self.userTable+ " where username=%s and study_name=%s"
-        vals = str(username), str(self.study_name)
-        try:
-            self.execute(qry, vals, commit=True)
-        except Exception as e:
-            raise Exception(e)
-
     def get_user_metadata(self, user_id: uuid = None, username: str = None) -> dict:
         """
         Get user metadata by user_id or by username
@@ -121,18 +103,14 @@ class UserHandler():
             raise ValueError("User ID/name cannot be empty.")
 
         if user_id and not username:
-            qry = "select user_metadata from user where user_id=%s and study_name=%s"
-            vals = str(user_id), self.study_name
+            user = self.session.query(User).filter((User.user_id == user_id) & (User.study_name==self.study_name)).first()
         elif not user_id and username:
-            qry = "select user_metadata from user where username=%s and study_name=%s"
-            vals = str(username), self.study_name
+            user = self.session.query(User).filter((User.username == username) & (User.study_name == self.study_name)).first()
         else:
-            qry = "select user_metadata from user where user_id=%s and username=%s and study_name=%s"
-            vals = str(user_id), str(username), self.study_name
+            user = self.session.query(User).filter((User.user_id == user_id) & (User.username == username) & (User.study_name == self.study_name)).first()
 
-        rows = self.execute(qry, vals)
-        if len(rows) > 0:
-            return rows[0]
+        if user:
+            return user.user_metadata
         else:
             return {}
 
@@ -159,18 +137,14 @@ class UserHandler():
             raise ValueError("User ID or auth token cannot be empty.")
 
         if username and not auth_token:
-            qry = "select user_id, username, user_settings from user where username=%s and study_name=%s"
-            vals = str(username), self.study_name
+            user = self.session.query(User.user_settings).filter((User.username == username) & (User.study_name == self.study_name)).first()
         elif not username and auth_token:
-            qry = "select user_id, username, user_settings from user where token=%s and study_name=%s"
-            vals = str(auth_token), self.study_name
+            user = self.session.query(User.user_settings).filter((User.token == auth_token) & (User.study_name == self.study_name)).first()
         else:
-            qry = "select user_id, username, user_settings from user where username=%s and token=%s and study_name=%s"
-            vals = str(username), str(auth_token), self.study_name
+            user = self.session.query(User.user_settings).filter((User.username == username) & (User.token == auth_token) & (User.study_name == self.study_name)).first()
 
-        rows = self.execute(qry, vals)
-        if len(rows) > 0:
-            return rows[0]
+        if user:
+            return user.user_settings
         else:
             return {}
 
@@ -197,10 +171,8 @@ class UserHandler():
         if encrypt_password:
             password = self.encrypt_user_password(password)
 
-        qry = "select * from user where username=%s and password=%s and study_name=%s"
-        vals = username, password, self.study_name
-
-        rows = self.execute(qry, vals)
+        user = self.session.query(User).filter(
+            (User.username == username) & (User.password==password) & (User.study_name == self.study_name)).first()
 
         token_issue_time = datetime.now()
         expires = timedelta(seconds=int(self.config["cc"]['auth_token_expire_time']))
@@ -208,7 +180,8 @@ class UserHandler():
 
         token = jwt.encode({'username': username, "token_expire_at":str(token_expiry), "token_issued_at":str(token_issue_time)}, self.config["cc"]["auth_encryption_key"], algorithm='HS256')
         token = token.decode("utf-8")
-        if len(rows) == 0:
+
+        if not user:
             return {"status":False, "auth_token": "", "msg":" Incorrect username and/or password."}
         elif not self.update_auth_token(username, token, token_issue_time, token_expiry):
             return {"status":False, "auth_token": "", "msg": "cannot update auth token."}
@@ -231,17 +204,15 @@ class UserHandler():
         if not auth_token:
             raise ValueError("Auth token cannot be null/empty.")
 
-        qry = "select * from user where token=%s and username=%s and study_name=%s"
-        vals = auth_token, username, self.study_name
+        user = self.session.query(User).filter(
+            (User.username == username) & (User.token == auth_token) & (User.study_name == self.study_name)).first()
 
-        rows = self.execute(qry, vals)
-
-        if len(rows) == 0:
+        if not user:
             return False
         elif not checktime:
             return True
         else:
-            token_expiry_time = rows[0]["token_expiry"]
+            token_expiry_time = user.token_expiry
             localtz = timezone(self.time_zone)
             token_expiry_time = localtz.localize(token_expiry_time)
 
@@ -250,7 +221,7 @@ class UserHandler():
             else:
                 return True
 
-    def list_users(self) -> List[dict]:
+    def list_users(self) -> List[list]:
         """
         Get a list of all users part of a study.
 
@@ -259,26 +230,21 @@ class UserHandler():
         Raises:
             ValueError: Study name is a requied field.
         Returns:
-            list[dict]: Returns empty list if there is no user associated to the study_name and/or study_name does not exist.
+            list[list]: Returns empty list if there is no user associated to the study_name and/or study_name does not exist.
         Examples:
             >>> CC = CerebralCortex("/directory/path/of/configs/")
             >>> CC.list_users("mperf")
             >>> [{"76cc444c-4fb8-776e-2872-9472b4e66b16": "nasir_ali"}] # [{user_id, user_name}]
         """
 
-        results = []
-        qry = 'SELECT user_id, username FROM ' + self.userTable +" where study_name=%(study_name)s"
-        vals = {"study_name": self.study_name}
-        rows = self.execute(qry, vals)
+        rows = self.session.query(User.user_id, User.username).filter(User.study_name == self.study_name).all()
 
-        if len(rows) == 0:
-            return []
+        if rows:
+            return rows
         else:
-            for row in rows:
-                results.append(row)
-            return results
+            return []
 
-    def get_user_name(self, user_id: str) -> str:
+    def get_username(self, user_id: str) -> str:
         """
         Get the user name linked to a user id.
 
@@ -290,56 +256,18 @@ class UserHandler():
             ValueError: User ID is a required field.
         Examples:
             >>> CC = CerebralCortex("/directory/path/of/configs/")
-            >>> CC.get_user_name("76cc444c-4fb8-776e-2872-9472b4e66b16")
+            >>> CC.get_username("76cc444c-4fb8-776e-2872-9472b4e66b16")
             >>> 'nasir_ali'
         """
         if not user_id:
             raise ValueError("User ID is a required field.")
 
-        qry = "select username from " + self.userTable + " where user_id = %s and study_name=%s"
-        vals = str(user_id), self.study_name
+        user = self.session.query(User.username).filter((User.user_id==user_id) & (User.study_name == self.study_name)).first()
 
-        rows = self.execute(qry, vals)
-
-        if len(rows) == 0:
-            return ""
+        if not user:
+            raise Exception(str(user_id)+" does not exist.")
         else:
-            return rows[0]["username"]
-
-    def is_user(self, user_id: uuid = None, user_name: uuid = None) -> bool:
-        """
-        Checks whether a user exists in the system. One of both parameters could be set to verify whether user exist.
-
-        Args:
-            user_id (str): id (uuid) of a user
-            user_name (str): username of a user
-        Returns:
-            bool: True if a user exists in the system or False otherwise.
-        Raises:
-            ValueError: Both user_id and user_name cannot be None or empty.
-        Examples:
-            >>> CC = CerebralCortex("/directory/path/of/configs/")
-            >>> CC.is_user(user_id="76cc444c-4fb8-776e-2872-9472b4e66b16")
-            >>> True
-        """
-        if user_id and user_name:
-            qry = "select username from " + self.userTable + " where user_id = %s and username=%s and study_name=%s"
-            vals = str(user_id), user_name, self.study_name
-        elif user_id and not user_name:
-            qry = "select username from " + self.userTable + " where user_id = %s and study_name=%s"
-            vals = str(user_id), self.study_name
-        elif not user_id and user_name:
-            qry = "select username from " + self.userTable + " where username = %s and study_name=%s"
-            vals = str(user_name), self.study_name
-        else:
-            raise ValueError("Both user_id and user_name cannot be None or empty.")
-
-        rows = self.execute(qry, vals)
-
-        if len(rows) > 0:
-            return True
-        else:
-            return False
+            return user.username
 
     def get_user_id(self, user_name: str) -> str:
         """
@@ -359,15 +287,45 @@ class UserHandler():
         if not user_name:
             raise ValueError("User name is a required field.")
 
-        qry = "select user_id from " + self.userTable + " where username = %s and study_name=%s"
-        vals = str(user_name), self.study_name
+        user = self.session.query(User.user_id).filter((User.username==user_name) & (User.study_name == self.study_name)).first()
 
-        rows = self.execute(qry, vals)
-
-        if len(rows) == 0:
-            return ""
+        if not user:
+            raise Exception(str(user_name)+ " does not exist.")
         else:
-            return rows[0]["user_id"]
+            return user.user_id
+
+    def is_user(self, user_id: uuid = None, user_name: uuid = None) -> bool:
+        """
+        Checks whether a user exists in the system. One of both parameters could be set to verify whether user exist.
+
+        Args:
+            user_id (str): id (uuid) of a user
+            user_name (str): username of a user
+        Returns:
+            bool: True if a user exists in the system or False otherwise.
+        Raises:
+            ValueError: Both user_id and user_name cannot be None or empty.
+        Examples:
+            >>> CC = CerebralCortex("/directory/path/of/configs/")
+            >>> CC.is_user(user_id="76cc444c-4fb8-776e-2872-9472b4e66b16")
+            >>> True
+        """
+        if user_id and user_name:
+            user = self.session.query(User.username).filter(
+                (User.user_id == user_id) & (User.username==user_name) & (User.study_name == self.study_name)).first()
+        elif user_id and not user_name:
+            user = self.session.query(User.username).filter(
+                (User.user_id == user_id) & (User.study_name == self.study_name)).first()
+        elif not user_id and user_name:
+            user = self.session.query(User.username).filter(
+                (User.username == user_name) & (User.study_name == self.study_name)).first()
+        else:
+            raise ValueError("Both user_id and user_name cannot be None or empty.")
+
+        if user:
+            return True
+        else:
+            return False
 
     def update_auth_token(self, username: str, auth_token: str, auth_token_issued_time: datetime,
                           auth_token_expiry_time: datetime) -> bool:
@@ -388,11 +346,11 @@ class UserHandler():
         if not auth_token and not auth_token_expiry_time and not auth_token_issued_time:
             raise ValueError("Auth token and auth-token issue/expiry time cannot be None/empty.")
 
-        qry = "UPDATE " + self.userTable + " set token=%s, token_issued=%s, token_expiry=%s where username=%s and study_name=%s"
-        vals = auth_token, auth_token_issued_time, auth_token_expiry_time, username, self.study_name
-
         try:
-            self.execute(qry, vals, commit=True)
+            self.session.query(User).filter(
+                (User.username == username) & (User.study_name == self.study_name)).update({User.token: auth_token,
+                                                                                        User.token_issued: auth_token_issued_time,
+                                                                                        User.token_expiry: auth_token_expiry_time})
             return True
         except:
             return False
